@@ -3,10 +3,11 @@ from dagster.core.host_representation.external_data import (
     ExternalPartitionConfigData,
     ExternalPartitionExecutionErrorData,
     ExternalPartitionNamesData,
+    ExternalPartitionSetExecutionParamData,
     ExternalPartitionTagsData,
 )
 from dagster.core.host_representation.handle import RepositoryHandle
-from dagster.grpc.types import PartitionArgs, PartitionNamesArgs
+from dagster.grpc.types import PartitionArgs, PartitionNamesArgs, PartitionSetExecutionParamArgs
 
 from .utils import execute_unary_api_cli_command
 
@@ -128,3 +129,50 @@ def sync_get_external_partition_tags_grpc(
         ),
         (ExternalPartitionTagsData, ExternalPartitionExecutionErrorData),
     )
+
+
+def sync_get_external_partition_set_execution_param_data(
+    repository_handle, partition_set_name, partition_names
+):
+    check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
+    check.str_param(partition_set_name, 'partition_set_name')
+    check.list_param(partition_names, 'partition_names', of_type=str)
+
+    repository_origin = repository_handle.get_origin()
+
+    return check.inst(
+        execute_unary_api_cli_command(
+            repository_origin.executable_path,
+            'partition_set_execution_param_data',
+            PartitionSetExecutionParamArgs(
+                repository_origin=repository_origin,
+                partition_set_name=partition_set_name,
+                partition_names=partition_names,
+            ),
+        ),
+        (ExternalPartitionSetExecutionParamData, ExternalPartitionExecutionErrorData),
+    )
+
+
+def sync_get_external_partition_set_execution_param_data_grpc(
+    api_client, repository_handle, partition_set_name, partition_names
+):
+    from dagster.grpc.client import DagsterGrpcClient
+
+    check.inst_param(api_client, 'api_client', DagsterGrpcClient)
+    check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
+    check.str_param(partition_set_name, 'partition_set_name')
+    check.list_param(partition_names, 'partition_names', of_type=str)
+
+    repository_origin = repository_handle.get_origin()
+
+    return check.inst(
+        api_client.external_partition_set_execution_params(
+            partition_set_execution_param_args=PartitionSetExecutionParamArgs(
+                repository_origin=repository_origin,
+                partition_set_name=partition_set_name,
+                partition_names=partition_names,
+            ),
+        ),
+        (ExternalPartitionSetExecutionParamData, ExternalPartitionExecutionErrorData),
+    )
