@@ -1,11 +1,12 @@
 import os
 import re
+import warnings
 from collections import namedtuple
 from enum import Enum
 
 from dagster import check
 from dagster.core.errors import DagsterInvalidAssetKey
-from dagster.serdes import whitelist_for_serdes
+from dagster.serdes import Persistable, whitelist_for_persistence
 
 from .utils import DEFAULT_OUTPUT
 
@@ -40,27 +41,31 @@ def validate_structured_asset_key(l):
     return l
 
 
-@whitelist_for_serdes
-class AssetKey(namedtuple('_AssetKey', 'path')):
-    ''' Object representing the structure of an asset key.  Takes in a sanitized string or list of
-    strings.
+@whitelist_for_persistence
+class AssetKey(namedtuple('_AssetKey', 'path'), Persistable):
+    ''' Object representing the structure of an asset key.  Takes in a sanitized string, list of
+    strings, or tuple of strings.
 
     Example usage:
+
         AssetKey('flat_asset_key')
+
         AssetKey(['parent', 'child', 'grandchild'])
 
+        AssetKey(('parent', 'child', 'grandchild'))
+
     Args:
-        asset_key_str_or_list (str|str[]): String or list of strings.  A list of strings represent
-            the hierarchical structure of the asset_key.
+        path (str|str[]|str()): String, list of strings, or tuple of strings.  A list of strings
+            represent the hierarchical structure of the asset_key.
     '''
 
     def __new__(cls, path=None):
         if check.is_str(path):
             path = [validate_asset_key_string(path)]
+        elif isinstance(path, list):
+            path = validate_structured_asset_key(check.list_param(path, 'path', of_type=str))
         else:
-            path = validate_structured_asset_key(
-                check.list_param(path, 'asset_key_str_or_list', of_type=str)
-            )
+            path = validate_structured_asset_key(check.tuple_param(path, 'path', of_type=str))
 
         return super(AssetKey, cls).__new__(cls, path=path)
 
@@ -96,8 +101,10 @@ class AssetKey(namedtuple('_AssetKey', 'path')):
         return None
 
 
-@whitelist_for_serdes
-class EventMetadataEntry(namedtuple('_EventMetadataEntry', 'label description entry_data')):
+@whitelist_for_persistence
+class EventMetadataEntry(
+    namedtuple('_EventMetadataEntry', 'label description entry_data'), Persistable
+):
     '''The standard structure for describing metadata for Dagster events.
 
     Lists of objects of this type can be passed as arguments to Dagster events and will be displayed
@@ -125,7 +132,7 @@ class EventMetadataEntry(namedtuple('_EventMetadataEntry', 'label description en
         :py:class:`TextMetadataEntryData`.
 
         Args:
-            text (str): The text of this metadata entry.
+            text (Optional[str]): The text of this metadata entry.
             label (str): Short display label for this metadata entry.
             description (Optional[str]): A human-readable description of this metadata entry.
         '''
@@ -137,7 +144,7 @@ class EventMetadataEntry(namedtuple('_EventMetadataEntry', 'label description en
         :py:class:`UrlMetadataEntryData`.
 
         Args:
-            url (str): The URL contained by this metadata entry.
+            url (Optional[str]): The URL contained by this metadata entry.
             label (str): Short display label for this metadata entry.
             description (Optional[str]): A human-readable description of this metadata entry.
         '''
@@ -149,7 +156,7 @@ class EventMetadataEntry(namedtuple('_EventMetadataEntry', 'label description en
         :py:class:`PathMetadataEntryData`.
 
         Args:
-            path (str): The path contained by this metadata entry.
+            path (Optional[str]): The path contained by this metadata entry.
             label (str): Short display label for this metadata entry.
             description (Optional[str]): A human-readable description of this metadata entry.
         '''
@@ -161,8 +168,8 @@ class EventMetadataEntry(namedtuple('_EventMetadataEntry', 'label description en
         :py:class:`PathMetadataEntryData`.
 
         Args:
-            path (str): The path contained by this metadata entry.
-            label (Optional[str]): Short display label for this metadata entry. Defaults to the
+            path (Optional[str]): The path contained by this metadata entry.
+            label (str): Short display label for this metadata entry. Defaults to the
                 base name of the path.
             description (Optional[str]): A human-readable description of this metadata entry.
         '''
@@ -176,7 +183,7 @@ class EventMetadataEntry(namedtuple('_EventMetadataEntry', 'label description en
         :py:class:`JsonMetadataEntryData`.
 
         Args:
-            data (Dict[str, Any]): The JSON data contained by this metadata entry.
+            data (Optional[Dict[str, Any]]): The JSON data contained by this metadata entry.
             label (str): Short display label for this metadata entry.
             description (Optional[str]): A human-readable description of this metadata entry.
         '''
@@ -187,7 +194,7 @@ class EventMetadataEntry(namedtuple('_EventMetadataEntry', 'label description en
         '''Static constructor for a metadata entry containing markdown.
 
         Args:
-            md_str (str): The markdown contained by this metadata entry.
+            md_str (Optional[str]): The markdown contained by this metadata entry.
             label (str): Short display label for this metadata entry.
             description (Optional[str]): A human-readable description of this metadata entry.
         '''
@@ -204,75 +211,91 @@ class EventMetadataEntry(namedtuple('_EventMetadataEntry', 'label description en
 
     @staticmethod
     def float(value, label, description=None):
-        check.float_param(value, 'value')
-        return EventMetadataEntry(label, description, FloatMetadataEntryData(value),)
+        '''Static constructor for a metadata entry containing float as
+        :py:class:`FloatMetadataEntryData`.
+
+        Args:
+            value (Optional[float]): The float value contained by this metadata entry.
+            label (str): Short display label for this metadata entry.
+            description (Optional[str]): A human-readable description of this metadata entry.
+        '''
+        check.opt_float_param(value, 'value')
+        return EventMetadataEntry(label, description, FloatMetadataEntryData(value))
 
 
-@whitelist_for_serdes
-class TextMetadataEntryData(namedtuple('_TextMetadataEntryData', 'text')):
+@whitelist_for_persistence
+class TextMetadataEntryData(namedtuple('_TextMetadataEntryData', 'text'), Persistable):
     '''Container class for text metadata entry data.
 
     Args:
-        text (str): The text data.
+        text (Optional[str]): The text data.
     '''
 
     def __new__(cls, text):
-        return super(TextMetadataEntryData, cls).__new__(cls, check.str_param(text, 'text'))
+        return super(TextMetadataEntryData, cls).__new__(
+            cls, check.opt_str_param(text, 'text', default='')
+        )
 
 
-@whitelist_for_serdes
-class UrlMetadataEntryData(namedtuple('_UrlMetadataEntryData', 'url')):
+@whitelist_for_persistence
+class UrlMetadataEntryData(namedtuple('_UrlMetadataEntryData', 'url'), Persistable):
     '''Container class for URL metadata entry data.
 
     Args:
-        url (str): The URL as a string.
+        url (Optional[str]): The URL as a string.
     '''
 
     def __new__(cls, url):
-        return super(UrlMetadataEntryData, cls).__new__(cls, check.str_param(url, 'url'))
+        return super(UrlMetadataEntryData, cls).__new__(
+            cls, check.opt_str_param(url, 'url', default='')
+        )
 
 
-@whitelist_for_serdes
-class PathMetadataEntryData(namedtuple('_PathMetadataEntryData', 'path')):
+@whitelist_for_persistence
+class PathMetadataEntryData(namedtuple('_PathMetadataEntryData', 'path'), Persistable):
     '''Container class for path metadata entry data.
 
     Args:
-        path (str): The path as a string.
+        path (Optional[str]): The path as a string.
     '''
 
     def __new__(cls, path):
-        return super(PathMetadataEntryData, cls).__new__(cls, check.str_param(path, 'path'))
+        return super(PathMetadataEntryData, cls).__new__(
+            cls, check.opt_str_param(path, 'path', default='')
+        )
 
 
-@whitelist_for_serdes
-class JsonMetadataEntryData(namedtuple('_JsonMetadataEntryData', 'data')):
+@whitelist_for_persistence
+class JsonMetadataEntryData(namedtuple('_JsonMetadataEntryData', 'data'), Persistable):
     '''Container class for JSON metadata entry data.
 
     Args:
-        data (Dict[str, Any]): The JSON data.
+        data (Optional[Dict[str, Any]]): The JSON data.
     '''
 
     def __new__(cls, data):
         return super(JsonMetadataEntryData, cls).__new__(
-            cls, check.dict_param(data, 'data', key_type=str)
+            cls, check.opt_dict_param(data, 'data', key_type=str)
         )
 
 
-@whitelist_for_serdes
-class MarkdownMetadataEntryData(namedtuple('_MarkdownMetadataEntryData', 'md_str')):
+@whitelist_for_persistence
+class MarkdownMetadataEntryData(namedtuple('_MarkdownMetadataEntryData', 'md_str'), Persistable):
     '''Container class for markdown metadata entry data.
 
     Args:
-        md_str (str): The markdown as a string.
+        md_str (Optional[str]): The markdown as a string.
     '''
 
     def __new__(cls, md_str):
-        return super(MarkdownMetadataEntryData, cls).__new__(cls, check.str_param(md_str, 'md_str'))
+        return super(MarkdownMetadataEntryData, cls).__new__(
+            cls, check.opt_str_param(md_str, 'md_str', default='')
+        )
 
 
-@whitelist_for_serdes
+@whitelist_for_persistence
 class PythonArtifactMetadataEntryData(
-    namedtuple('_PythonArtifactMetadataEntryData', 'module name')
+    namedtuple('_PythonArtifactMetadataEntryData', 'module name'), Persistable
 ):
     def __new__(cls, module, name):
         return super(PythonArtifactMetadataEntryData, cls).__new__(
@@ -280,11 +303,13 @@ class PythonArtifactMetadataEntryData(
         )
 
 
-@whitelist_for_serdes
-class FloatMetadataEntryData(namedtuple('_FloatMetadataEntryData', 'value')):
+@whitelist_for_persistence
+class FloatMetadataEntryData(namedtuple('_FloatMetadataEntryData', 'value'), Persistable):
     def __new__(cls, value):
-        check.float_param(value, 'value')
-        return super(FloatMetadataEntryData, cls).__new__(cls, check.float_param(value, 'value'))
+        check.opt_float_param(value, 'value')
+        return super(FloatMetadataEntryData, cls).__new__(
+            cls, check.opt_float_param(value, 'value')
+        )
 
 
 EntryDataUnion = (
@@ -319,9 +344,75 @@ class Output(namedtuple('_Output', 'value output_name')):
         return super(Output, cls).__new__(cls, value, check.str_param(output_name, 'output_name'))
 
 
-@whitelist_for_serdes
+@whitelist_for_persistence
+class AssetMaterialization(
+    namedtuple('_AssetMaterialization', 'asset_key description metadata_entries'), Persistable
+):
+    '''Event indicating that a solid has materialized an asset.
+
+    Solid compute functions may yield events of this type whenever they wish to indicate to the
+    Dagster framework (and the end user) that they have produced a materialized value as a
+    side effect of computation. Unlike outputs, asset materializations can not be passed to other
+    solids, and their persistence is controlled by solid logic, rather than by the Dagster
+    framework.
+
+    Solid authors should use these events to organize metadata about the side effects of their
+    computations, enabling tooling like the Assets dashboard in `dagit`.
+
+    Args:
+        asset_key (str|List[str]|AssetKey): A key to identify the materialized asset across pipeline
+            runs
+        description (Optional[str]): A longer human-radable description of the materialized value.
+        metadata_entries (Optional[List[EventMetadataEntry]]): Arbitrary metadata about the
+            materialized value.
+    '''
+
+    def __new__(cls, asset_key, description=None, metadata_entries=None):
+        if isinstance(asset_key, AssetKey):
+            check.inst_param(asset_key, 'asset_key', AssetKey)
+        elif check.is_str(asset_key):
+            asset_key = AssetKey(parse_asset_key_string(asset_key))
+        elif isinstance(asset_key, list):
+            check.is_list(asset_key, of_type=str)
+            asset_key = AssetKey(asset_key)
+        else:
+            check.is_tuple(asset_key, of_type=str)
+            asset_key = AssetKey(asset_key)
+
+        return super(AssetMaterialization, cls).__new__(
+            cls,
+            asset_key=asset_key,
+            description=check.opt_str_param(description, 'description'),
+            metadata_entries=check.opt_list_param(
+                metadata_entries, metadata_entries, of_type=EventMetadataEntry
+            ),
+        )
+
+    @property
+    def label(self):
+        return self.asset_key.to_string()
+
+    @staticmethod
+    def file(path, description=None, asset_key=None):
+        '''Static constructor for standard materializations corresponding to files on disk.
+
+        Args:
+            path (str): The path to the file.
+            description (Optional[str]): A human-readable description of the materialization.
+        '''
+        if not asset_key:
+            asset_key = path
+
+        return AssetMaterialization(
+            asset_key=asset_key,
+            description=description,
+            metadata_entries=[EventMetadataEntry.fspath(path)],
+        )
+
+
+@whitelist_for_persistence
 class Materialization(
-    namedtuple('_Materialization', 'label description metadata_entries asset_key')
+    namedtuple('_Materialization', 'label description metadata_entries asset_key'), Persistable
 ):
     '''Event indicating that a solid has materialized a value.
 
@@ -342,6 +433,40 @@ class Materialization(
             across pipeline runs
     '''
 
+    def __new__(
+        cls,
+        label=None,
+        description=None,
+        metadata_entries=None,
+        asset_key=None,
+        skip_deprecation_warning=False,
+    ):
+        if asset_key and check.is_str(asset_key):
+            asset_key = AssetKey(parse_asset_key_string(asset_key))
+        else:
+            check.opt_inst_param(asset_key, 'asset_key', AssetKey)
+
+        if not label:
+            check.param_invariant(
+                asset_key and asset_key.path,
+                'label',
+                'Either label or asset_key with a path must be provided',
+            )
+            label = asset_key.to_string()
+
+        if not skip_deprecation_warning:
+            warnings.warn('`Materialization` is deprecated; use `AssetMaterialization` instead.')
+
+        return super(Materialization, cls).__new__(
+            cls,
+            label=check.str_param(label, 'label'),
+            description=check.opt_str_param(description, 'description'),
+            metadata_entries=check.opt_list_param(
+                metadata_entries, metadata_entries, of_type=EventMetadataEntry
+            ),
+            asset_key=asset_key,
+        )
+
     @staticmethod
     def file(path, description=None, asset_key=None):
         '''Static constructor for standard materializations corresponding to files on disk.
@@ -357,34 +482,16 @@ class Materialization(
             asset_key=asset_key,
         )
 
-    def __new__(cls, label=None, description=None, metadata_entries=None, asset_key=None):
-        if asset_key and check.is_str(asset_key):
-            asset_key = AssetKey(parse_asset_key_string(asset_key))
-        else:
-            check.opt_inst_param(asset_key, 'asset_key', AssetKey)
-
-        if not label:
-            check.param_invariant(
-                asset_key and asset_key.path,
-                'label',
-                'Either label or asset_key with a path must be provided',
-            )
-            label = asset_key.to_string()
-
-        return super(Materialization, cls).__new__(
-            cls,
-            label=check.str_param(label, 'label'),
-            description=check.opt_str_param(description, 'description'),
-            metadata_entries=check.opt_list_param(
-                metadata_entries, metadata_entries, of_type=EventMetadataEntry
-            ),
-            asset_key=asset_key,
-        )
+    @classmethod
+    def from_storage_dict(cls, storage_dict):
+        # override the default `from_storage_dict` implementation in order to skip the deprecation
+        # warning for historical Materialization events, loaded from event_log storage
+        return Materialization.__new__(cls, skip_deprecation_warning=True, **storage_dict)
 
 
-@whitelist_for_serdes
+@whitelist_for_persistence
 class ExpectationResult(
-    namedtuple('_ExpectationResult', 'success label description metadata_entries')
+    namedtuple('_ExpectationResult', 'success label description metadata_entries'), Persistable
 ):
     '''Event corresponding to a data quality test.
 
@@ -412,8 +519,8 @@ class ExpectationResult(
         )
 
 
-@whitelist_for_serdes
-class TypeCheck(namedtuple('_TypeCheck', 'success description metadata_entries')):
+@whitelist_for_persistence
+class TypeCheck(namedtuple('_TypeCheck', 'success description metadata_entries'), Persistable):
     '''Event corresponding to a successful typecheck.
 
     Events of this type should be returned by user-defined type checks when they need to encapsulate
@@ -548,3 +655,20 @@ class ObjectStoreOperation(
                 **kwargs
             )
         )
+
+
+class HookExecutionResult(namedtuple('_HookExecutionResult', 'hook_name is_skipped')):
+    '''This event is used internally to indicate the execution result of a hook, e.g. whether the
+    user-defined hook function is skipped.
+
+    Args:
+        hook_name (str): The name of the hook.
+        is_skipped (bool): ``False`` if the hook_fn is executed, ``True`` otheriwse.
+    '''
+
+    def __new__(cls, hook_name, is_skipped=None):
+        return super(HookExecutionResult, cls).__new__(
+            cls,
+            hook_name=check.str_param(hook_name, 'hook_name'),
+            is_skipped=check.opt_bool_param(is_skipped, 'is_skipped', default=False),
+        )
