@@ -256,19 +256,18 @@ class DagsterConfigMappingFunctionError(DagsterUserCodeExecutionError):
     '''
 
 
-class DagsterInputHydrationConfigError(DagsterUserCodeExecutionError):
+class DagsterTypeLoadingError(DagsterUserCodeExecutionError):
     '''
-    Indicates that an unexpected error occurred while executing the body of an input hydration
-    config function defined in a :py:class:`~dagster.InputHydrationConfig` during input hydration of
-    a custom type.
+    Indicates that an unexpected error occurred while executing the body of an type load
+    function defined in a :py:class:`~dagster.DagsterTypeLoader` during loading of a custom type.
     '''
 
 
-class DagsterOutputMaterializationError(DagsterUserCodeExecutionError):
+class DagsterTypeMaterializationError(DagsterUserCodeExecutionError):
     '''
     Indicates that an unexpected error occurred while executing the body of an output
-    materialization function defined in a :py:class:`~dagster.OutputMaterializationConfig` during
-    output materialization of a custom type.
+    materialization function defined in a :py:class:`~dagster.DagsterTypeMaterializer` during
+    materialization of a custom type.
     '''
 
 
@@ -283,7 +282,7 @@ class DagsterUnknownResourceError(DagsterError, AttributeError):
     def __init__(self, resource_name, *args, **kwargs):
         self.resource_name = check.str_param(resource_name, 'resource_name')
         msg = (
-            'Unknown resource `{resource_name}. Specify `{resource_name}` as a required resource '
+            'Unknown resource `{resource_name}`. Specify `{resource_name}` as a required resource '
             'on the compute / config function that accessed it.'
         ).format(resource_name=resource_name)
         super(DagsterUnknownResourceError, self).__init__(msg, *args, **kwargs)
@@ -335,10 +334,50 @@ class DagsterSubprocessError(DagsterError):
         super(DagsterSubprocessError, self).__init__(*args, **kwargs)
 
 
+class DagsterUserCodeProcessError(DagsterError):
+    '''An exception has occurred in a user code process that the host process raising this error
+     was communicating with.'''
+
+    def __init__(self, *args, **kwargs):
+        from dagster.utils.error import SerializableErrorInfo
+
+        self.user_code_process_error_infos = check.list_param(
+            kwargs.pop('user_code_process_error_infos'),
+            'user_code_process_error_infos',
+            SerializableErrorInfo,
+        )
+        super(DagsterUserCodeProcessError, self).__init__(*args, **kwargs)
+
+
 class DagsterLaunchFailedError(DagsterError):
     '''Indicates an error while attempting to launch a pipeline run.
     '''
 
+    def __init__(self, *args, **kwargs):
+        from dagster.utils.error import SerializableErrorInfo
+
+        self.serializable_error_info = check.opt_inst_param(
+            kwargs.pop('serializable_error_info', None),
+            'serializable_error_info',
+            SerializableErrorInfo,
+        )
+        super(DagsterLaunchFailedError, self).__init__(*args, **kwargs)
+
+
+class DagsterBackfillFailedError(DagsterError):
+    '''Indicates an error while attempting to launch a backfill.
+    '''
+
+    def __init__(self, *args, **kwargs):
+        from dagster.utils.error import SerializableErrorInfo
+
+        self.serializable_error_info = check.opt_inst_param(
+            kwargs.pop('serializable_error_info', None),
+            'serializable_error_info',
+            SerializableErrorInfo,
+        )
+        super(DagsterBackfillFailedError, self).__init__(*args, **kwargs)
+
 
 class DagsterInstanceMigrationRequired(DagsterError):
     '''Indicates that the dagster instance must be migrated.'''
@@ -402,7 +441,7 @@ class DagsterEventLogInvalidForRun(DagsterError):
 
 
 class ScheduleExecutionError(DagsterUserCodeExecutionError):
-    '''Errors raised during the execution of schedule.'''
+    '''Errors raised in a user process during the execution of schedule.'''
 
 
 class PartitionExecutionError(DagsterUserCodeExecutionError):
@@ -411,3 +450,11 @@ class PartitionExecutionError(DagsterUserCodeExecutionError):
 
 class DagsterInvalidAssetKey(DagsterUserCodeExecutionError):
     ''' Error raised by invalid asset key '''
+
+
+class HookExecutionError(DagsterUserCodeExecutionError):
+    ''' Error raised during the execution of a user-defined hook. '''
+
+
+class DagsterImportError(DagsterError):
+    ''' Import error raised while importing user-code. '''
