@@ -14,21 +14,27 @@ from dagster.core.system_config.objects import EnvironmentConfig
 from dagster.core.telemetry import log_repo_stats, telemetry_wrapper
 from dagster.core.utils import str_format_set
 from dagster.utils import merge_dicts
-from dagster.utils.backcompat import canonicalize_run_config
 
-from .context_creation_pipeline import pipeline_initialization_manager, scoped_pipeline_context
+from .context_creation_pipeline import (
+    ExecutionContextManager,
+    PipelineExecutionContextManager,
+    PlanExecutionContextManager,
+    scoped_pipeline_context,
+)
 from .results import PipelineExecutionResult
 
 ## Brief guide to the execution APIs
-# | function name             | operates over      | sync  | supports    | creates new PipelineRun |
-# |                           |                    |       | reexecution | in instance             |
-# | ------------------------- | ------------------ | ----- | ----------- | ----------------------- |
-# | execute_pipeline_iterator | ExecutablePipeline | async | no          | yes                     |
-# | execute_pipeline          | ExecutablePipeline | sync  | no          | yes                     |
-# | execute_run_iterator      | PipelineRun        | async | (1)         | no                      |
-# | execute_run               | PipelineRun        | sync  | (1)         | no                      |
-# | execute_plan_iterator     | ExecutionPlan      | async | (2)         | no                      |
-# | execute_plan              | ExecutionPlan      | sync  | (2)         | no                      |
+# | function name               | operates over      | sync  | supports    | creates new PipelineRun |
+# |                             |                    |       | reexecution | in instance             |
+# | --------------------------- | ------------------ | ----- | ----------- | ----------------------- |
+# | execute_pipeline_iterator   | ExecutablePipeline | async | no          | yes                     |
+# | execute_pipeline            | ExecutablePipeline | sync  | no          | yes                     |
+# | execute_run_iterator        | PipelineRun        | async | (1)         | no                      |
+# | execute_run                 | PipelineRun        | sync  | (1)         | no                      |
+# | execute_plan_iterator       | ExecutionPlan      | async | (2)         | no                      |
+# | execute_plan                | ExecutionPlan      | sync  | (2)         | no                      |
+# | reexecute_pipeline          | ExecutablePipeline | sync  | yes         | yes                     |
+# | reexecute_pipeline_iterator | ExecutablePipeline | async | yes         | yes                     |
 #
 # Notes on reexecution support:
 # (1) The appropriate bits must be set on the PipelineRun passed to this function. Specifically,
@@ -63,7 +69,7 @@ def execute_run_iterator(pipeline, pipeline_run, instance):
             )
     execution_plan = create_execution_plan(
         pipeline,
-        environment_dict=pipeline_run.environment_dict,
+        run_config=pipeline_run.run_config,
         mode=pipeline_run.mode,
         step_keys_to_execute=pipeline_run.step_keys_to_execute,
     )
@@ -71,12 +77,14 @@ def execute_run_iterator(pipeline, pipeline_run, instance):
     return iter(
         _ExecuteRunWithPlanIterable(
             execution_plan=execution_plan,
-            pipeline_run=pipeline_run,
-            instance=instance,
             iterator=_pipeline_execution_iterator,
-            environment_dict=pipeline_run.environment_dict,
-            retries=None,
-            raise_on_error=False,
+            execution_context_manager=PipelineExecutionContextManager(
+                execution_plan=execution_plan,
+                pipeline_run=pipeline_run,
+                instance=instance,
+                run_config=pipeline_run.run_config,
+                raise_on_error=False,
+            ),
         )
     )
 
@@ -130,19 +138,21 @@ def execute_run(pipeline, pipeline_run, instance, raise_on_error=False):
 
     execution_plan = create_execution_plan(
         pipeline,
-        environment_dict=pipeline_run.environment_dict,
+        run_config=pipeline_run.run_config,
         mode=pipeline_run.mode,
         step_keys_to_execute=pipeline_run.step_keys_to_execute,
     )
 
     _execute_run_iterable = _ExecuteRunWithPlanIterable(
         execution_plan=execution_plan,
-        pipeline_run=pipeline_run,
-        instance=instance,
         iterator=_pipeline_execution_iterator,
-        environment_dict=pipeline_run.environment_dict,
-        retries=None,
-        raise_on_error=raise_on_error,
+        execution_context_manager=PipelineExecutionContextManager(
+            execution_plan=execution_plan,
+            pipeline_run=pipeline_run,
+            instance=instance,
+            run_config=pipeline_run.run_config,
+            raise_on_error=raise_on_error,
+        ),
     )
     event_list = list(_execute_run_iterable)
     pipeline_context = _execute_run_iterable.pipeline_context
@@ -153,11 +163,12 @@ def execute_run(pipeline, pipeline_run, instance, raise_on_error=False):
         event_list,
         lambda: scoped_pipeline_context(
             execution_plan,
-            pipeline_run.environment_dict,
+            pipeline_run.run_config,
             pipeline_run,
             instance,
+            intermediate_storage=pipeline_context.intermediate_storage,
             system_storage_data=SystemStorageData(
-                intermediates_manager=pipeline_context.intermediates_manager,
+                intermediate_storage=pipeline_context.intermediate_storage,
                 file_manager=pipeline_context.file_manager,
             ),
         ),
@@ -172,7 +183,6 @@ def execute_pipeline_iterator(
     tags=None,
     solid_selection=None,
     instance=None,
-    environment_dict=None,
 ):
     '''Execute a pipeline iteratively.
 
@@ -185,7 +195,7 @@ def execute_pipeline_iterator(
 
     Parameters:
         pipeline (Union[ExecutablePipeline, PipelineDefinition]): The pipeline to execute.
-        environment_dict (Optional[dict]): The environment configuration that parametrizes this run,
+        run_config (Optional[dict]): The environment configuration that parametrizes this run,
             as a dict.
         mode (Optional[str]): The name of the pipeline mode to use. You may not set both ``mode``
             and ``preset``.
@@ -207,12 +217,10 @@ def execute_pipeline_iterator(
     Returns:
       Iterator[DagsterEvent]: The stream of events resulting from pipeline execution.
     '''
-    # stack level is to punch through helper function
-    environment_dict = canonicalize_run_config(run_config, environment_dict, stacklevel=4)
 
     (
         pipeline,
-        environment_dict,
+        run_config,
         instance,
         mode,
         tags,
@@ -220,7 +228,7 @@ def execute_pipeline_iterator(
         solid_selection,
     ) = _check_execute_pipeline_args(
         pipeline=pipeline,
-        environment_dict=environment_dict,
+        run_config=run_config,
         mode=mode,
         preset=preset,
         tags=tags,
@@ -230,7 +238,7 @@ def execute_pipeline_iterator(
 
     pipeline_run = instance.create_run_for_pipeline(
         pipeline_def=pipeline.get_definition(),
-        environment_dict=environment_dict,
+        run_config=run_config,
         mode=mode,
         solid_selection=solid_selection,
         solids_to_execute=solids_to_execute,
@@ -240,7 +248,6 @@ def execute_pipeline_iterator(
     return execute_run_iterator(pipeline, pipeline_run, instance)
 
 
-@telemetry_wrapper
 def execute_pipeline(
     pipeline,
     run_config=None,
@@ -250,7 +257,6 @@ def execute_pipeline(
     solid_selection=None,
     instance=None,
     raise_on_error=True,
-    environment_dict=None,
 ):
     '''Execute a pipeline synchronously.
 
@@ -259,7 +265,7 @@ def execute_pipeline(
 
     Parameters:
         pipeline (Union[ExecutablePipeline, PipelineDefinition]): The pipeline to execute.
-        environment_dict (Optional[dict]): The environment configuration that parametrizes this run,
+        run_config (Optional[dict]): The environment configuration that parametrizes this run,
             as a dict.
         mode (Optional[str]): The name of the pipeline mode to use. You may not set both ``mode``
             and ``preset``.
@@ -288,12 +294,34 @@ def execute_pipeline(
     This is the entrypoint for dagster CLI execution. For the dagster-graphql entrypoint, see
     ``dagster.core.execution.api.execute_plan()``.
     '''
-    # stack level is to punch through helper function and telemetry wrapper
-    environment_dict = canonicalize_run_config(run_config, environment_dict, stacklevel=5)
+    instance = instance or DagsterInstance.ephemeral()
+    return _logged_execute_pipeline(
+        pipeline,
+        run_config=run_config,
+        mode=mode,
+        preset=preset,
+        tags=tags,
+        solid_selection=solid_selection,
+        instance=instance,
+        raise_on_error=raise_on_error,
+    )
+
+
+@telemetry_wrapper
+def _logged_execute_pipeline(
+    pipeline,
+    run_config=None,
+    mode=None,
+    preset=None,
+    tags=None,
+    solid_selection=None,
+    instance=None,
+    raise_on_error=True,
+):
 
     (
         pipeline,
-        environment_dict,
+        run_config,
         instance,
         mode,
         tags,
@@ -301,7 +329,7 @@ def execute_pipeline(
         solid_selection,
     ) = _check_execute_pipeline_args(
         pipeline=pipeline,
-        environment_dict=environment_dict,
+        run_config=run_config,
         mode=mode,
         preset=preset,
         tags=tags,
@@ -313,7 +341,7 @@ def execute_pipeline(
 
     pipeline_run = instance.create_run_for_pipeline(
         pipeline_def=pipeline.get_definition(),
-        environment_dict=environment_dict,
+        run_config=run_config,
         mode=mode,
         solid_selection=solid_selection,
         solids_to_execute=solids_to_execute,
@@ -323,30 +351,178 @@ def execute_pipeline(
     return execute_run(pipeline, pipeline_run, instance, raise_on_error=raise_on_error)
 
 
+def reexecute_pipeline(
+    pipeline,
+    parent_run_id,
+    run_config=None,
+    step_keys_to_execute=None,
+    mode=None,
+    preset=None,
+    tags=None,
+    instance=None,
+    raise_on_error=True,
+):
+    '''Reexecute an existing pipeline run.
+
+    Users will typically call this API when testing pipeline reexecution, or running standalone
+    scripts.
+
+    Parameters:
+        pipeline (Union[ExecutablePipeline, PipelineDefinition]): The pipeline to execute.
+        parent_run_id (str): The id of the previous run to reexecute. The run must exist in the
+            instance.
+        run_config (Optional[dict]): The environment configuration that parametrizes this run,
+            as a dict.
+        step_keys_to_execute (Optional[List[str]]): Keys of the steps to execute.
+        mode (Optional[str]): The name of the pipeline mode to use. You may not set both ``mode``
+            and ``preset``.
+        preset (Optional[str]): The name of the pipeline preset to use. You may not set both
+            ``mode`` and ``preset``.
+        tags (Optional[Dict[str, Any]]): Arbitrary key-value pairs that will be added to pipeline
+            logs.
+        instance (Optional[DagsterInstance]): The instance to execute against. If this is ``None``,
+            an ephemeral instance will be used, and no artifacts will be persisted from the run.
+        raise_on_error (Optional[bool]): Whether or not to raise exceptions when they occur.
+            Defaults to ``True``, since this is the most useful behavior in test.
+
+    Returns:
+      :py:class:`PipelineExecutionResult`: The result of pipeline execution.
+
+    For the asynchronous version, see :py:func:`reexecute_pipeline_iterator`.
+    '''
+    check.str_param(parent_run_id, 'parent_run_id')
+
+    (pipeline, run_config, instance, mode, tags, _, _) = _check_execute_pipeline_args(
+        pipeline=pipeline,
+        run_config=run_config,
+        mode=mode,
+        preset=preset,
+        tags=tags,
+        instance=instance,
+    )
+
+    parent_pipeline_run = instance.get_run_by_id(parent_run_id)
+    check.invariant(
+        parent_pipeline_run,
+        'No parent run with id {parent_run_id} found in instance.'.format(
+            parent_run_id=parent_run_id
+        ),
+    )
+
+    pipeline_run = instance.create_run_for_pipeline(
+        pipeline_def=pipeline.get_definition(),
+        run_config=run_config,
+        mode=mode,
+        tags=tags,
+        solid_selection=parent_pipeline_run.solid_selection,
+        solids_to_execute=parent_pipeline_run.solids_to_execute,
+        step_keys_to_execute=step_keys_to_execute,
+        root_run_id=parent_pipeline_run.root_run_id or parent_pipeline_run.run_id,
+        parent_run_id=parent_pipeline_run.run_id,
+    )
+
+    return execute_run(pipeline, pipeline_run, instance, raise_on_error=raise_on_error)
+
+
+def reexecute_pipeline_iterator(
+    pipeline,
+    parent_run_id,
+    run_config=None,
+    step_keys_to_execute=None,
+    mode=None,
+    preset=None,
+    tags=None,
+    instance=None,
+):
+    '''Reexecute a pipeline iteratively.
+
+    Rather than package up the result of running a pipeline into a single object, like
+    :py:func:`reexecute_pipeline`, this function yields the stream of events resulting from pipeline
+    reexecution.
+
+    This is intended to allow the caller to handle these events on a streaming basis in whatever
+    way is appropriate.
+
+    Parameters:
+        pipeline (Union[ExecutablePipeline, PipelineDefinition]): The pipeline to execute.
+        parent_run_id (str): The id of the previous run to reexecute. The run must exist in the
+            instance.
+        run_config (Optional[dict]): The environment configuration that parametrizes this run,
+            as a dict.
+        step_keys_to_execute (Optional[List[str]]): Keys of the steps to execute.
+        mode (Optional[str]): The name of the pipeline mode to use. You may not set both ``mode``
+            and ``preset``.
+        preset (Optional[str]): The name of the pipeline preset to use. You may not set both
+            ``mode`` and ``preset``.
+        tags (Optional[Dict[str, Any]]): Arbitrary key-value pairs that will be added to pipeline
+            logs.
+        instance (Optional[DagsterInstance]): The instance to execute against. If this is ``None``,
+            an ephemeral instance will be used, and no artifacts will be persisted from the run.
+
+    Returns:
+      Iterator[DagsterEvent]: The stream of events resulting from pipeline reexecution.
+    '''
+    check.str_param(parent_run_id, 'parent_run_id')
+
+    (pipeline, run_config, instance, mode, tags, _, _) = _check_execute_pipeline_args(
+        pipeline=pipeline,
+        run_config=run_config,
+        mode=mode,
+        preset=preset,
+        tags=tags,
+        instance=instance,
+        solid_selection=None,
+    )
+    parent_pipeline_run = instance.get_run_by_id(parent_run_id)
+    check.invariant(
+        parent_pipeline_run,
+        'No parent run with id {parent_run_id} found in instance.'.format(
+            parent_run_id=parent_run_id
+        ),
+    )
+
+    pipeline_run = instance.create_run_for_pipeline(
+        pipeline_def=pipeline.get_definition(),
+        run_config=run_config,
+        mode=mode,
+        tags=tags,
+        solid_selection=parent_pipeline_run.solid_selection,
+        solids_to_execute=parent_pipeline_run.solids_to_execute,
+        step_keys_to_execute=step_keys_to_execute,
+        root_run_id=parent_pipeline_run.root_run_id or parent_pipeline_run.run_id,
+        parent_run_id=parent_pipeline_run.run_id,
+    )
+
+    return execute_run_iterator(pipeline, pipeline_run, instance)
+
+
 def execute_plan_iterator(
-    execution_plan, pipeline_run, instance, retries=None, environment_dict=None,
+    execution_plan, pipeline_run, instance, retries=None, run_config=None,
 ):
     check.inst_param(execution_plan, 'execution_plan', ExecutionPlan)
     check.inst_param(pipeline_run, 'pipeline_run', PipelineRun)
     check.inst_param(instance, 'instance', DagsterInstance)
     retries = check.opt_inst_param(retries, 'retries', Retries, Retries.disabled_mode())
-    environment_dict = check.opt_dict_param(environment_dict, 'environment_dict')
+    run_config = check.opt_dict_param(run_config, 'run_config')
 
     return iter(
         _ExecuteRunWithPlanIterable(
             execution_plan=execution_plan,
-            environment_dict=environment_dict,
-            pipeline_run=pipeline_run,
-            instance=instance,
-            retries=retries,
             iterator=inner_plan_execution_iterator,
-            raise_on_error=False,
+            execution_context_manager=PlanExecutionContextManager(
+                retries=retries,
+                execution_plan=execution_plan,
+                run_config=run_config,
+                pipeline_run=pipeline_run,
+                instance=instance,
+                raise_on_error=False,
+            ),
         )
     )
 
 
 def execute_plan(
-    execution_plan, instance, pipeline_run, environment_dict=None, retries=None,
+    execution_plan, instance, pipeline_run, run_config=None, retries=None,
 ):
     '''This is the entry point of dagster-graphql executions. For the dagster CLI entry point, see
     execute_pipeline() above.
@@ -354,13 +530,13 @@ def execute_plan(
     check.inst_param(execution_plan, 'execution_plan', ExecutionPlan)
     check.inst_param(instance, 'instance', DagsterInstance)
     check.inst_param(pipeline_run, 'pipeline_run', PipelineRun)
-    environment_dict = check.opt_dict_param(environment_dict, 'environment_dict')
+    run_config = check.opt_dict_param(run_config, 'run_config')
     check.opt_inst_param(retries, 'retries', Retries)
 
     return list(
         execute_plan_iterator(
             execution_plan=execution_plan,
-            environment_dict=environment_dict,
+            run_config=run_config,
             pipeline_run=pipeline_run,
             instance=instance,
             retries=retries,
@@ -377,62 +553,76 @@ def _check_pipeline(pipeline):
     return pipeline
 
 
-def create_execution_plan(pipeline, environment_dict=None, mode=None, step_keys_to_execute=None):
+def create_execution_plan(pipeline, run_config=None, mode=None, step_keys_to_execute=None):
     pipeline = _check_pipeline(pipeline)
     pipeline_def = pipeline.get_definition()
     check.inst_param(pipeline_def, 'pipeline_def', PipelineDefinition)
 
-    environment_dict = check.opt_dict_param(environment_dict, 'environment_dict', key_type=str)
+    run_config = check.opt_dict_param(run_config, 'run_config', key_type=str)
     mode = check.opt_str_param(mode, 'mode', default=pipeline_def.get_default_mode_name())
     check.opt_list_param(step_keys_to_execute, 'step_keys_to_execute', of_type=str)
 
-    environment_config = EnvironmentConfig.build(pipeline_def, environment_dict, mode=mode)
+    environment_config = EnvironmentConfig.build(pipeline_def, run_config, mode=mode)
 
     return ExecutionPlan.build(
         pipeline, environment_config, mode=mode, step_keys_to_execute=step_keys_to_execute
     )
 
 
-def _pipeline_execution_iterator(pipeline_context, execution_plan, retries=None):
+class BoolRef:
+    def __init__(self, value):
+        self.value = value
+
+
+def _core_execution_iterator(pipeline_context, execution_plan, steps_started, pipeline_success_ref):
+    try:
+        for event in pipeline_context.executor.execute(pipeline_context, execution_plan):
+            if event.is_step_start:
+                steps_started.add(event.step_key)
+            if event.is_step_success:
+                if event.step_key not in steps_started:
+                    pipeline_success_ref.value = False
+                else:
+                    steps_started.remove(event.step_key)
+            if event.is_step_failure:
+                pipeline_success_ref.value = False
+            yield event
+    except (Exception, KeyboardInterrupt):
+        pipeline_success_ref.value = False
+        raise  # finally block will run before this is re-raised
+
+
+def _pipeline_execution_iterator(pipeline_context, execution_plan):
     '''A complete execution of a pipeline. Yields pipeline start, success,
     and failure events.
 
     Args:
         pipeline_context (SystemPipelineExecutionContext):
         execution_plan (ExecutionPlan):
-        retries (None): Must be None. This is to align the signature of
-            `_pipeline_execution_iterator` with that of
-            `dagster.core.execution.plan.execute_plan.inner_plan_execution_iterator` so the same
-            machinery in _ExecuteRunWithPlanIterable can call them without unpleasant workarounds.
-            (Default: None)
     '''
     check.inst_param(pipeline_context, 'pipeline_context', SystemPipelineExecutionContext)
     check.inst_param(execution_plan, 'execution_plan', ExecutionPlan)
-    check.invariant(
-        retries is None, 'Programming error: Retries not supported in _pipeline_execution_iterator'
-    )
+
     yield DagsterEvent.pipeline_start(pipeline_context)
 
-    pipeline_success = True
+    steps_started = set([])
+    pipeline_success_ref = BoolRef(True)
     generator_closed = False
     try:
-        for event in pipeline_context.executor_config.get_engine().execute(
-            pipeline_context, execution_plan
+        for event in _core_execution_iterator(
+            pipeline_context, execution_plan, steps_started, pipeline_success_ref
         ):
-            if event.is_step_failure:
-                pipeline_success = False
             yield event
     except GeneratorExit:
         # Shouldn't happen, but avoid runtime-exception in case this generator gets GC-ed
         # (see https://amir.rachum.com/blog/2017/03/03/generator-cleanup/).
         generator_closed = True
-        pipeline_success = False
+        pipeline_success_ref.value = False
         raise
-    except (Exception, KeyboardInterrupt):
-        pipeline_success = False
-        raise  # finally block will run before this is re-raised
     finally:
-        if pipeline_success:
+        if steps_started:
+            pipeline_success_ref.value = False
+        if pipeline_success_ref.value:
             event = DagsterEvent.pipeline_success(pipeline_context)
         else:
             event = DagsterEvent.pipeline_failure(pipeline_context)
@@ -449,46 +639,25 @@ class _ExecuteRunWithPlanIterable(object):
     cf. `dagster.utils.EventGenerationManager`.
     '''
 
-    def __init__(
-        self,
-        execution_plan,
-        pipeline_run,
-        instance,
-        iterator,
-        environment_dict,
-        retries,
-        raise_on_error,
-    ):
+    def __init__(self, execution_plan, iterator, execution_context_manager):
         self.execution_plan = check.inst_param(execution_plan, 'execution_plan', ExecutionPlan)
-        self.pipeline_run = check.inst_param(pipeline_run, 'pipeline_run', PipelineRun)
-        self.instance = check.inst_param(instance, 'instance', DagsterInstance)
         self.iterator = check.callable_param(iterator, 'iterator')
-        self.environment_dict = (
-            check.opt_dict_param(environment_dict, 'environment_dict')
-            or self.pipeline_run.environment_dict
+        self.execution_context_manager = check.inst_param(
+            execution_context_manager, 'execution_context_manager', ExecutionContextManager
         )
-        self.retries = check.opt_inst_param(retries, 'retries', Retries)
-        self.raise_on_error = check.bool_param(raise_on_error, 'raise_on_error')
+
         self.pipeline_context = None
 
     def __iter__(self):
-        initialization_manager = pipeline_initialization_manager(
-            self.execution_plan,
-            self.environment_dict,
-            self.pipeline_run,
-            self.instance,
-            raise_on_error=self.raise_on_error,
-        )
-        for event in initialization_manager.generate_setup_events():
+
+        for event in self.execution_context_manager.prepare_context():
             yield event
-        self.pipeline_context = initialization_manager.get_object()
+        self.pipeline_context = self.execution_context_manager.get_context()
         generator_closed = False
         try:
             if self.pipeline_context:  # False if we had a pipeline init failure
                 for event in self.iterator(
-                    execution_plan=self.execution_plan,
-                    pipeline_context=self.pipeline_context,
-                    retries=self.retries,
+                    execution_plan=self.execution_plan, pipeline_context=self.pipeline_context,
                 ):
                     yield event
         except GeneratorExit:
@@ -497,19 +666,19 @@ class _ExecuteRunWithPlanIterable(object):
             generator_closed = True
             raise
         finally:
-            for event in initialization_manager.generate_teardown_events():
+            for event in self.execution_context_manager.shutdown_context():
                 if not generator_closed:
                     yield event
 
 
 def _check_execute_pipeline_args(
-    pipeline, environment_dict, mode, preset, tags, solid_selection, instance
+    pipeline, run_config, mode, preset, tags, instance, solid_selection=None
 ):
     pipeline = _check_pipeline(pipeline)
     pipeline_def = pipeline.get_definition()
     check.inst_param(pipeline_def, 'pipeline_def', PipelineDefinition)
 
-    environment_dict = check.opt_dict_param(environment_dict, 'environment_dict')
+    run_config = check.opt_dict_param(run_config, 'run_config')
     check.opt_str_param(mode, 'mode')
     check.opt_str_param(preset, 'preset')
     check.invariant(
@@ -527,12 +696,12 @@ def _check_execute_pipeline_args(
 
         if pipeline_preset.run_config is not None:
             check.invariant(
-                (not environment_dict) or (pipeline_preset.run_config == environment_dict),
+                (not run_config) or (pipeline_preset.run_config == run_config),
                 'The environment set in preset \'{preset}\' does not agree with the environment '
-                'passed in the `environment_dict` argument.'.format(preset=preset),
+                'passed in the `run_config` argument.'.format(preset=preset),
             )
 
-            environment_dict = pipeline_preset.run_config
+            run_config = pipeline_preset.run_config
 
         # load solid_selection from preset
         if pipeline_preset.solid_selection is not None:
@@ -557,6 +726,8 @@ def _check_execute_pipeline_args(
 
         mode = pipeline_preset.mode
 
+        tags = merge_dicts(pipeline_preset.tags, tags)
+
     if mode is not None:
         if not pipeline_def.has_mode_definition(mode):
             raise DagsterInvariantViolationError(
@@ -589,7 +760,7 @@ def _check_execute_pipeline_args(
 
     return (
         pipeline,
-        environment_dict,
+        run_config,
         instance,
         mode,
         tags,
