@@ -1,14 +1,16 @@
 from dagster import check
 from dagster.config import Field
-from dagster.core.definitions.intermediate_storage import intermediate_storage
+from dagster.core.definitions.intermediate_storage import (
+    intermediate_storage as intermediate_storage_fn,
+)
 from dagster.core.definitions.system_storage import SystemStorageData, system_storage
 from dagster.core.storage.type_storage import TypeStoragePluginRegistry
 
 from .file_manager import LocalFileManager
 from .init import InitIntermediateStorageContext
-from .intermediate_store import IntermediateStore, build_fs_intermediate_store
-from .intermediates_manager import (
-    IntermediateStoreIntermediatesManager,
+from .intermediate_storage import (
+    ObjectStoreIntermediateStorage,
+    build_fs_intermediate_storage,
     build_in_mem_intermediates_storage,
 )
 from .object_store import FilesystemObjectStore, InMemoryObjectStore, ObjectStore
@@ -16,7 +18,7 @@ from .object_store import FilesystemObjectStore, InMemoryObjectStore, ObjectStor
 
 def create_mem_system_storage_data(init_context):
     return SystemStorageData(
-        intermediates_manager=build_in_mem_intermediates_storage(
+        intermediate_storage=build_in_mem_intermediates_storage(
             init_context.pipeline_run.run_id,
             type_storage_plugin_registry=init_context.type_storage_plugin_registry,
         ),
@@ -41,7 +43,7 @@ def build_intermediate_storage_from_object_store(
     root_for_run_id = check.callable_param(root_for_run_id, 'root_for_run_id')
     init_context = check.inst_param(init_context, 'init_context', InitIntermediateStorageContext)
 
-    intermediate_store = IntermediateStore(
+    return ObjectStoreIntermediateStorage(
         object_store=object_store,
         run_id=init_context.pipeline_run.run_id,
         root_for_run_id=root_for_run_id,
@@ -49,7 +51,6 @@ def build_intermediate_storage_from_object_store(
         if init_context.type_storage_plugin_registry
         else TypeStoragePluginRegistry(types_to_register=[]),
     )
-    return IntermediateStoreIntermediatesManager(intermediate_store)
 
 
 @system_storage(name='in_memory', is_persistent=False, required_resource_keys=set())
@@ -68,7 +69,7 @@ def mem_system_storage(init_context):
     return create_mem_system_storage_data(init_context)
 
 
-@intermediate_storage(name='in_memory', is_persistent=False, required_resource_keys=set())
+@intermediate_storage_fn(name='in_memory', is_persistent=False, required_resource_keys=set())
 def mem_intermediate_storage(init_context):
     '''The default in-memory intermediate storage.
 
@@ -110,7 +111,7 @@ def fs_system_storage(init_context):
     override_dir = init_context.system_storage_config.get('base_dir')
     if override_dir:
         file_manager = LocalFileManager(override_dir)
-        intermediate_store = build_fs_intermediate_store(
+        intermediate_storage = build_fs_intermediate_storage(
             root_for_run_id=lambda _: override_dir,
             run_id=init_context.pipeline_run.run_id,
             type_storage_plugin_registry=init_context.type_storage_plugin_registry,
@@ -119,19 +120,16 @@ def fs_system_storage(init_context):
         file_manager = LocalFileManager.for_instance(
             init_context.instance, init_context.pipeline_run.run_id
         )
-        intermediate_store = build_fs_intermediate_store(
+        intermediate_storage = build_fs_intermediate_storage(
             init_context.instance.intermediates_directory,
             run_id=init_context.pipeline_run.run_id,
             type_storage_plugin_registry=init_context.type_storage_plugin_registry,
         )
 
-    return SystemStorageData(
-        file_manager=file_manager,
-        intermediates_manager=IntermediateStoreIntermediatesManager(intermediate_store),
-    )
+    return SystemStorageData(file_manager=file_manager, intermediate_storage=intermediate_storage,)
 
 
-@intermediate_storage(
+@intermediate_storage_fn(
     name='filesystem',
     is_persistent=True,
     config_schema={'base_dir': Field(str, is_required=False)},
