@@ -1,5 +1,5 @@
 '''As an open source project, we collect usage statistics to inform development priorities.
-For more information, check out the docs at https://docs.dagster.io/docs/install/telemetry/'
+For more information, check out the docs at https://docs.dagster.io/install/telemetry/'
 
 To see the logs we send, inspect $DAGSTER_HOME/logs/ if $DAGSTER_HOME is set or ~/.dagster/logs/
 
@@ -30,9 +30,9 @@ import yaml
 from dagster import check
 from dagster.core.definitions.executable import ExecutablePipeline
 from dagster.core.definitions.reconstructable import (
-    EPHEMERAL_NAME,
     ReconstructablePipeline,
     ReconstructableRepository,
+    get_ephemeral_repository_name,
 )
 from dagster.core.errors import DagsterInvariantViolationError
 from dagster.core.instance import DagsterInstance
@@ -48,11 +48,11 @@ START_DAGIT_WEBSERVER = 'start_dagit_webserver'
 TELEMETRY_VERSION = '0.2'
 
 # When adding to TELEMETRY_WHITELISTED_FUNCTIONS, please also update the literalinclude in
-# docs/next/src/pages/docs/install/telemetry.mdx
+# docs/next/src/pages/install/index.mdx
 TELEMETRY_WHITELISTED_FUNCTIONS = {
-    'pipeline_execute_command',
-    'pipeline_launch_command',
-    'execute_pipeline',
+    'execute_execute_command',
+    '_logged_execute_pipeline',
+    'execute_launch_command',
 }
 
 
@@ -60,6 +60,9 @@ def telemetry_wrapper(f):
     '''
     Wrapper around functions that are logged. Will log the function_name, client_time, and
     elapsed_time, and success.
+
+    Wrapped function must be in the list of whitelisted function, and must have a DagsterInstance
+    parameter named 'instance' in the signature.
     '''
     if f.__name__ not in TELEMETRY_WHITELISTED_FUNCTIONS:
         raise DagsterInvariantViolationError(
@@ -69,13 +72,24 @@ def telemetry_wrapper(f):
             )
         )
 
+    var_names = f.__code__.co_varnames
+    try:
+        instance_index = var_names.index('instance')
+    except ValueError:
+        raise DagsterInvariantViolationError(
+            'Attempted to log telemetry for function {name} that does not take a DagsterInstance '
+            'in a parameter called \'instance\''
+        )
+
     @wraps(f)
     def wrap(*args, **kwargs):
+        instance = _check_telemetry_instance_param(args, kwargs, instance_index)
         start_time = datetime.datetime.now()
-        log_action(action=f.__name__ + '_started', client_time=start_time)
+        log_action(instance=instance, action=f.__name__ + '_started', client_time=start_time)
         result = f(*args, **kwargs)
         end_time = datetime.datetime.now()
         log_action(
+            instance=instance,
             action=f.__name__ + '_ended',
             client_time=end_time,
             elapsed_time=end_time - start_time,
@@ -216,6 +230,26 @@ def get_log_queue_dir():
     return dagster_home_logs_queue_path
 
 
+def _check_telemetry_instance_param(args, kwargs, instance_index):
+    if 'instance' in kwargs:
+        return check.inst_param(
+            kwargs['instance'],
+            'instance',
+            DagsterInstance,
+            '\'instance\' parameter passed as keyword argument must be a DagsterInstance',
+        )
+    else:
+        check.invariant(len(args) > instance_index)
+        return check.inst_param(
+            args[instance_index],
+            'instance',
+            DagsterInstance,
+            '\'instance\' argument at position {position} must be a DagsterInstance'.format(
+                position=instance_index
+            ),
+        )
+
+
 def _get_telemetry_logger():
     logger = logging.getLogger('dagster_telemetry_logger')
 
@@ -236,8 +270,9 @@ def write_telemetry_log_line(log_line):
     logger.info(json.dumps(log_line))
 
 
-def _get_instance_telemetry_info():
-    dagster_telemetry_enabled = _get_instance_telemetry_enabled(DagsterInstance.get())
+def _get_instance_telemetry_info(instance):
+    check.inst_param(instance, 'instance', DagsterInstance)
+    dagster_telemetry_enabled = _get_instance_telemetry_enabled(instance)
     instance_id = None
     if dagster_telemetry_enabled:
         instance_id = _get_or_set_instance_id()
@@ -290,6 +325,38 @@ def hash_name(name):
     return hashlib.sha256(name.encode('utf-8')).hexdigest()
 
 
+def log_external_repo_stats(instance, source, external_repo, external_pipeline=None):
+    from dagster.core.host_representation.external import (
+        ExternalPipeline,
+        ExternalRepository,
+    )
+
+    check.inst_param(instance, 'instance', DagsterInstance)
+    check.str_param(source, 'source')
+    check.inst_param(external_repo, 'external_repo', ExternalRepository)
+    check.opt_inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
+
+    if _get_instance_telemetry_enabled(instance):
+        instance_id = _get_or_set_instance_id()
+
+        pipeline_name_hash = hash_name(external_pipeline.name) if external_pipeline else ''
+        repo_hash = hash_name(external_repo.name)
+        num_pipelines_in_repo = len(external_repo.get_all_external_pipelines())
+
+        write_telemetry_log_line(
+            TelemetryEntry(
+                action=UPDATE_REPO_STATS,
+                client_time=str(datetime.datetime.now()),
+                event_id=str(uuid.uuid4()),
+                instance_id=instance_id,
+                pipeline_name_hash=pipeline_name_hash,
+                num_pipelines_in_repo=str(num_pipelines_in_repo),
+                repo_hash=repo_hash,
+                metadata={'source': source},
+            )._asdict()
+        )
+
+
 def log_repo_stats(instance, source, pipeline=None, repo=None):
     check.inst_param(instance, 'instance', DagsterInstance)
     check.str_param(source, 'source')
@@ -311,7 +378,7 @@ def log_repo_stats(instance, source, pipeline=None, repo=None):
             num_pipelines_in_repo = len(repository.pipeline_names)
         else:
             pipeline_name_hash = hash_name(pipeline.get_definition().name)
-            repo_hash = hash_name(EPHEMERAL_NAME)
+            repo_hash = hash_name(get_ephemeral_repository_name(pipeline.get_definition().name))
             num_pipelines_in_repo = 1
 
         write_telemetry_log_line(
@@ -328,8 +395,12 @@ def log_repo_stats(instance, source, pipeline=None, repo=None):
         )
 
 
-def log_action(action, client_time=datetime.datetime.now(), elapsed_time=None, metadata=None):
-    (dagster_telemetry_enabled, instance_id) = _get_instance_telemetry_info()
+def log_action(instance, action, client_time=None, elapsed_time=None, metadata=None):
+    check.inst_param(instance, 'instance', DagsterInstance)
+    if client_time is None:
+        client_time = datetime.datetime.now()
+
+    (dagster_telemetry_enabled, instance_id) = _get_instance_telemetry_info(instance)
 
     if dagster_telemetry_enabled:
         # Log general statistics
@@ -349,7 +420,7 @@ TELEMETRY_TEXT = '''
   %(telemetry)s
 
   As an open source project, we collect usage statistics to inform development priorities. For more
-  information, read https://docs.dagster.io/docs/install/telemetry.
+  information, read https://docs.dagster.io/install/telemetry.
 
   We will not see or store solid definitions, pipeline definitions, modes, resources, context, or
   any data that is processed within solids and pipelines.
