@@ -441,8 +441,27 @@ class _Nothing(DagsterType):
 class PythonObjectDagsterType(DagsterType):
     '''Define a type in dagster whose typecheck is an isinstance check.
 
+    Specifically, the type can either be a single python type (e.g. int),
+    or a tuple of types (e.g. (int, float)) which is treated as a union.
+
+    Examples:
+        .. code-block:: python
+            ntype = PythonObjectDagsterType(python_type=int)
+            assert ntype.name == 'int'
+            assert_success(ntype, 1)
+            assert_failure(ntype, 'a')
+        ```
+
+        .. code-block:: python
+            ntype = PythonObjectDagsterType(python_type=(int, float))
+            assert ntype.name == 'Union[int, float]'
+            assert_success(ntype, 1)
+            assert_success(ntype, 1.5)
+            assert_failure(ntype, 'a')
+
+
     Args:
-        python_type (Type): The dagster typecheck function calls instanceof on
+        python_type (Union[Type, Tuple[Type, ...]): The dagster typecheck function calls instanceof on
             this type.
         name (Optional[str]): Name the type. Defaults to the name of ``python_type``.
         key (Optional[str]): Key of the type. Defaults to name.
@@ -472,8 +491,17 @@ class PythonObjectDagsterType(DagsterType):
     '''
 
     def __init__(self, python_type, key=None, name=None, **kwargs):
-        self.python_type = check.type_param(python_type, 'python_type')
-        name = check.opt_str_param(name, 'name', python_type.__name__)
+        if isinstance(python_type, tuple):
+            self.python_type = check.tuple_param(
+                python_type, 'python_type', of_type=tuple(check.type_types for item in python_type)
+            )
+            self.type_str = 'Union[{}]'.format(
+                ', '.join(python_type.__name__ for python_type in python_type)
+            )
+        else:
+            self.python_type = check.type_param(python_type, 'python_type')
+            self.type_str = python_type.__name__
+        name = check.opt_str_param(name, 'name', self.type_str)
         key = check.opt_str_param(key, 'key', name)
         super(PythonObjectDagsterType, self).__init__(
             key=key, name=name, type_check_fn=self.type_check_method, **kwargs
@@ -486,76 +514,6 @@ class PythonObjectDagsterType(DagsterType):
                 description=(
                     'Value of type {value_type} failed type check for Dagster type {dagster_type}, '
                     'expected value to be of Python type {expected_type}.'
-                ).format(
-                    value_type=type(value),
-                    dagster_type=self.name,
-                    expected_type=self.python_type.__name__,
-                ),
-            )
-
-        return TypeCheck(success=True)
-
-
-class PythonUnionDagsterType(DagsterType):
-    '''Define a type in dagster whose typecheck is an isinstance check over a union defined as a tuple of types.
-
-    Args:
-        python_types (Tuple[Type]): The dagster typecheck function calls instanceof on
-            this type tuple.
-        name (Optional[str]): Name the type. Defaults to "Union[```(type.__name__ for type in python_types)```]".
-        key (Optional[str]): Key of the type. Defaults to name.
-        description (Optional[str]): A markdown-formatted string, displayed in tooling.
-        loader (Optional[DagsterTypeLoader]): An instance of a class that
-            inherits from :py:class:`~dagster.DagsterTypeLoader` and can map config data to a value of
-            this type. Specify this argument if you will need to shim values of this type using the
-            config machinery. As a rule, you should use the
-            :py:func:`@dagster_type_loader <dagster.dagster_type_loader>` decorator to construct
-            these arguments.
-        materializer (Optional[DagsterTypeMaterializer]): An instance of a class
-            that inherits from :py:class:`~dagster.DagsterTypeMaterializer` and can persist values of
-            this type. As a rule, you should use the
-            :py:func:`@dagster_type_mate <dagster.dagster_type_mate>`
-            decorator to construct these arguments.
-        serialization_strategy (Optional[SerializationStrategy]): An instance of a class that
-            inherits from :py:class:`SerializationStrategy`. The default strategy for serializing
-            this value when automatically persisting it between execution steps. You should set
-            this value if the ordinary serialization machinery (e.g., pickle) will not be adequate
-            for this type.
-        auto_plugins (Optional[List[Type[TypeStoragePlugin]]]): If types must be serialized differently
-            depending on the storage being used for intermediates, they should specify this
-            argument. In these cases the serialization_strategy argument is not sufficient because
-            serialization requires specialized API calls, e.g. to call an S3 API directly instead
-            of using a generic file object. See ``dagster_pyspark.DataFrame`` for an example.
-
-    '''
-
-    def __init__(self, python_types, key=None, name=None, description=None, **kwargs):
-        # ugly workaround to handle unions in tuples
-        self.python_types = check.tuple_param(
-            python_types, 'python_types', of_type=tuple(check.type_types for item in python_types)
-        )
-        self.type_str = 'Union[{}]'.format(
-            ', '.join(python_type.__name__ for python_type in python_types)
-        )
-        name = check.opt_str_param(name, 'name', self.type_str)
-        key = check.opt_str_param(key, 'key', name)
-        description = check.opt_str_param(description, 'description', name)
-
-        super(PythonUnionDagsterType, self).__init__(
-            key=key,
-            name=name,
-            description=description,
-            type_check_fn=self.type_check_method,
-            **kwargs
-        )
-
-    def type_check_method(self, _context, value):
-        if not isinstance(value, self.python_types):
-            return TypeCheck(
-                success=False,
-                description=(
-                    'Value of type {value_type} failed type check for Dagster type {dagster_type}, '
-                    'expected value to be within Python type {expected_type}.'
                 ).format(
                     value_type=type(value), dagster_type=self.name, expected_type=self.type_str,
                 ),
